// WRTui begin

function WRTui(){
    this.init();
}

WRTui.prototype.currentView = null;

WRTui.prototype.init = function(){
    this.views = new Array();
	
    var imageFrames = new Array('WRTui/Resources/loader_f01.png', 'WRTui/Resources/loader_f02.png', 'WRTui/Resources/loader_f03.png', 'WRTui/Resources/loader_f04.png', 'WRTui/Resources/loader_f05.png', 'WRTui/Resources/loader_f06.png', 'WRTui/Resources/loader_f07.png', 'WRTui/Resources/loader_f08.png');
    this.alertAnimator = new MultipleImagesAnimator('', imageFrames, 100);
    
    this.alertElement = document.createElement('div');
    this.alertElement.className = 'UIAlert';
    this.hideAlert();
    document.body.appendChild(this.alertElement);
}

WRTui.prototype.showView = function(viewObj){
    if (isSet(viewObj)) {
        if (this.currentView != null) {
            this.currentView.hide();
            this.currentView = viewObj; // this.views[id];
            this.currentView.show();
            window.scroll(0, 0);
        }
        else {
            this.currentView = viewObj; // this.views[id];
            this.currentView.show();
            window.scroll(0, 0);
        }
    }
    else {
        if (this.currentView != null) {
            this.currentView.show();
        }
    }
}

WRTui.prototype.hideView = function(){
    if (this.currentView != null) {
        this.currentView.hide();
    }
}

WRTui.prototype.showAlert = function(message, duration){
    this.alertElement.className = 'UIAlert';
    
    this.alertElement.style.height = document.documentElement.clientHeight;
    this.alertElement.innerHTML = '<div id="alertText" class="UIAlertText"><img id="alertImage" src="WRTui/Resources/loader_f01.png" align="right">' + message + '</div>';
    this.alertAnimator.imageElement = document.getElementById('alertImage');
    this.alertAnimator.start();
    var yPos = (window.innerHeight / 2) - (parseInt(document.getElementById('alertText').offsetHeight) / 2);
    var xPos = (window.innerWidth / 2) - (parseInt(document.getElementById('alertText').offsetWidth) / 2);
    
    var alertText = document.getElementById('alertText');
    alertText.style.top = parseInt(yPos + document.body.scrollTop) + 'px';
    alertText.style.left = parseInt(xPos) + 'px';
    if (isSet(duration)) {
        setTimeout(this.hideThisAlert(this), duration);
    }
}

WRTui.prototype.hideAlert = function(){
    this.alertAnimator.stop();
    this.alertElement.className = 'hidden';    
}

WRTui.prototype.hideThisAlert = function(selfReference){
    return (function(){
        selfReference.alertAnimator.stop();
        selfReference.alertElement.className = 'hidden';
    });
    
}

// WRTui end

// UIView begin

function UIView(id, captionString){
    if (isSet(id)) {
        this.init(id, captionString);
    }
}

UIView.prototype.id = null;

UIView.prototype.parentElement = null;
UIView.prototype.captionElement = null;
UIView.prototype.elements = null;

UIView.prototype.init = function(id, captionString){
    this.id = id;
    this.elements = new Array();
    
    this.parentElement = document.getElementById(id); // document.createElement('div');
    this.parentElement.className = 'UIView';
    this.hide();

    this.setCaption(captionString);
    
    this.parentElement.appendChild(this.captionElement);
    document.body.appendChild(this.parentElement);	
}

UIView.prototype.addControl = function(controlObj){
    this.parentElement.appendChild(controlObj.parentElement);
    this.elements.push(controlObj);
}

UIView.prototype.show = function(){
    this.parentElement.className = 'UIView';
}

UIView.prototype.hide = function(){
    this.parentElement.className = 'UIView hidden';
}

UIView.prototype.reset = function(captionString){
    this.parentElement.innerHTML = '';
	if (isSet(captionString)) this.setCaption(captionString);
    this.parentElement.appendChild(this.captionElement);
    
    for (i = 0; i < this.elements.length; i++) {
        this.elements[i].destroy();
    }
    
    this.elements = new Array();
}

UIView.prototype.setCaption = function(captionString){
    this.captionElement = document.createElement('div');
    this.captionElement.className = 'UIViewCaption';
    this.captionElement.innerHTML = '<img src="WRTui/Resources/icon-mini.png" align="absmiddle">' + captionString;
}

// UIView end


// UITable begin

function UITable(id, header, footer, tableStyle){
    if (isSet(id)) {
        if (isSet(tableStyle)) {
            tableStyle = ' ' + tableStyle;
        }
        else {
            tableStyle = '';
        }
        this.init(id, header, footer, tableStyle);
    }
}

UITable.prototype.id = null;

UITable.prototype.tableStyle = null;

UITable.prototype.header = null;
UITable.prototype.footer = null;

UITable.prototype.parentElement = null;
UITable.prototype.rootElement = null;

UITable.prototype.elements = null;


UITable.prototype.init = function(id, header, footer, tableStyle){
    this.id = id;
    this.parentElement = document.createElement('ul');
    this.rootElement = this.parentElement;
    this.parentElement.className = 'UITable' + tableStyle;
    this.rowElements = new Array();
    this.elements = new Array();
    
    if (header != null) {
        this.header = header;
        this.addHeader(header);
    }
    if (footer != null) {
        this.footer = footer;
        this.addFooter(footer);
    }
}

UITable.prototype.destroy = function(){
    this.rootElement.innerHTML = '';
    this.id = null;
    this.tableStyle = null;
    this.header = null;
    this.footer = null;
    this.parentElement = null;
    this.rootElement = null;
	this.rowElements = null;
    this.elements = null;
}

UITable.prototype.removeRow = function(rowIndex) {
	tmpSplice = this.rowElements.splice(rowIndex,1);
	this.parentElement.removeChild(tmpSplice[0]);
}

UITable.prototype.addHTMLElement = function(rowText){
    rowElement = document.createElement('li');
    rowElement.innerHTML = rowText;
    //    rowElement.className = 'UITableRowHeader';
    this.parentElement.appendChild(rowElement);
    this.rowElements.push(rowElement);
}

UITable.prototype.addHeader = function(rowText){
    rowElement = document.createElement('li');
    rowElement.innerHTML = rowText;
    rowElement.className = 'UITableRowHeader';
    this.parentElement.appendChild(rowElement);
    this.rowElements.push(rowElement);
}

UITable.prototype.addFooter = function(rowText){
    rowElement = document.createElement('li');
    rowElement.innerHTML = rowText;
    rowElement.className = 'UITableRowFooter';
    this.parentElement.appendChild(rowElement);
    this.rowElements.push(rowElement);
}

UITable.prototype.addDivider = function(){
    rowElement = document.createElement('li');
    rowElement.innerHTML = '&nbsp';
    rowElement.className = 'UITableRowDivider';
    this.parentElement.appendChild(rowElement);
    this.rowElements.push(rowElement);
}

UITable.prototype.addPanel = function(panelText){
    rowElement = document.createElement('li');
    rowElement.innerHTML = panelText;
    rowElement.className = 'UITableRowPanel';
    this.parentElement.appendChild(rowElement);
}

UITable.prototype.addRow = function(rowText, rowEventHandler){
    rowElement = document.createElement('li');
    rowElement.innerHTML = rowText;
    if (rowEventHandler != undefined) {
//        rowElement.addEventListener('click', rowEventHandler, true);
        /*
        // Byt ut ovanstående click event handler mot nedanstående ...
        // Kod för att anropa en funktion som kan koppla ihop eventen med objektet som skapats så att man kan komma åt objekt.properties -- self = objektet som skapats
		// Vad är fördelen med att göra detta? Man kan ändra stylesheeten på klickade kontrollen så att man kan visuellt visa att man tryckt på knappen
        this.buttonElement.addEventListener("click", function(event) {
        self.controlClicked(event);
        event.stopPropagation();
        event.preventDefault();
        }, true);
        */
        rowElement.addEventListener("click", function(event) {
			eventObject = event.currentTarget;
			event.currentTarget.className = 'UITableRowSelected';
			setTimeout( function() {
				eventObject.className = 'UITableRow';
			}, 500);
			rowEventHandler(event);
			event.stopPropagation();
			event.preventDefault();
        }, true);
        
    }
    
    if (this.header != null && this.rowElements.length == 0) {
        rowElement.className = 'UITableRowHeader';
    }
    else {
        rowElement.className = 'UITableRowFooter';
    }
    
    if (this.footer != null) {
        if (this.parentElement.childNodes.length > 0) 
            rowElement.className = 'UITableRow';
        this.parentElement.insertBefore(rowElement, this.parentElement.lastChild);
    }
    else {
        if (this.parentElement.childNodes.length > 0) {
            if (this.parentElement.childNodes[this.parentElement.childNodes.length - 1].className == 'UITableRowFooter') {
                this.parentElement.childNodes[this.parentElement.childNodes.length - 1].className = 'UITableRow';
            }
        }
        this.parentElement.appendChild(rowElement);
        this.rowElements.push(rowElement);
        return rowElement;
    }
}

UITable.prototype.addFormInputText = function(inputID, value, rowEventHandler){

    // Create the input text item
    inputElement = document.createElement('input');
    inputElement.className = 'inputText';
    inputElement.type = 'text';
    inputElement.value = value;
    if (isSet(inputID)) 
        inputElement.id = inputID;
    
    // Create the input row
    rowElement = document.createElement('li');
    rowElement.className = 'liFormInputL';
    rowElement.appendChild(inputElement);
    
    if (isSet(rowEventHandler)) 
        rowElement.addEventListener('focus', rowEventHandler, false);
    
    this.parentElement.appendChild(rowElement);
    
    // Create the input custom box "cap" on the right side
    rowElement = document.createElement('li');
    rowElement.innerHTML = '&nbsp;';
    rowElement.className = 'liFormInputR';
    
    this.parentElement.appendChild(rowElement);
    
    this.elements[inputID] = inputElement;
    
}

UITable.prototype.addFormButton = function(rowText, rowClass, rowEventHandler){

    // Create the form button item
    rowElementL = document.createElement('li');
    
    if (rowText != null) {
        rowElementL.innerHTML = rowText;
    }
    else {
        rowElementL.innerHTML = '&nbsp;';
    }
    
    if (rowClass != null) {
        rowElementL.className = rowClass;
    }
    else {
        rowElementL.className = 'liFormButtonL';
    }
    
    rowElementL.addEventListener('click', rowEventHandler, false);
    
    this.parentElement.appendChild(rowElementL);
    
    if (rowText != null) {
        // Create the input custom bow "cap" on the right side
        rowElementR = document.createElement('li');
        rowElementR.innerHTML = '&nbsp;';
        rowElementR.className = 'liFormButtonR';
        
        rowElementR.addEventListener('click', rowEventHandler, false);
        
        this.parentElement.appendChild(rowElementR);
    }
    
    return rowElementL;
}

UITable.prototype.newFormSelect = function(selectID){

    selectElement = document.createElement('select');
    selectElement.id = selectID;
    
    this.elements[selectID] = selectElement;
    
}

UITable.prototype.addFormSelect = function(selectID, header, rowEventHandler){
    // Create the header if that's anything else than null
    if (header != null) {
        rowElement = document.createElement('li');
        rowElement.innerHTML = header;
        rowElement.className = 'liFormSelectHeader';
        
        this.parentElement.appendChild(rowElement);
    }
    
    // Create the select item
    rowElement = document.createElement('li');
	if (header == null) {
		rowElement.className = 'liFormSelectNoHeader';
	} else {
		rowElement.className = 'liFormSelect';			
	}
	
    if (isSet(rowEventHandler)) 
        rowElement.addEventListener('change', rowEventHandler, false);
    
    // Add the select element from the select elements array
    rowElement.appendChild(this.elements[selectID]);
    
    this.parentElement.appendChild(rowElement);
    
    // Create the select custom box "cap" on the right side
    rowElement = document.createElement('li');
    rowElement.innerHTML = '&nbsp;';
	if (header == null) {
		rowElement.className = 'liFormSelectRNoHeader';
	} else {
		rowElement.className = 'liFormSelectR';
	}
    
    this.parentElement.appendChild(rowElement);
    
}

UITable.prototype.addFormOption = function(selectID, value, text, selected){
    optionElement = document.createElement('option');
    optionElement.value = value;
    optionElement.text = text;
    
    this.elements[selectID].appendChild(optionElement);
    if (isSet(selected)) 
        this.elements[selectID].selectedIndex = this.elements[selectID].childNodes.length - 1;
}


// UITable end

// Animator class start
function MultipleImagesAnimator(imageElement, imagePaths, frameInterval){
    this.imageElement = imageElement;
    this.imagePaths = imagePaths;
    this.frameInterval = frameInterval;
    this.currentFrameIndex = 0;
    this.interval = null;
}

MultipleImagesAnimator.prototype.start = function(){
    if (this.interval == null) {
        var self = this;
        
        this.interval = setInterval(function(){
            self.nextFrame();
        }, this.frameInterval);
    }
}


MultipleImagesAnimator.prototype.stop = function(){
    if (this.interval != null) {
        clearInterval(this.interval);
        
        this.interval = null;
    }
}

MultipleImagesAnimator.prototype.nextFrame = function(){
    this.imageElement.src = this.imagePaths[this.currentFrameIndex];
    
    this.currentFrameIndex = (this.currentFrameIndex + 1) % this.imagePaths.length;
}


// Animator class end


//================================RECOMMEND CAROUSEL================================START
function RecommendCarouselDIV(){
	this.refreshTimer;
    this.xOffset = 0;
    this.destinationX = this.xOffset;
    this.parentElement = document.createElement('div');
    this.parentElement.className = "RecommendedCarouselItemDIV";
}

RecommendCarouselDIV.prototype.getRootElement = function(){
    return this.parentElement;
}

RecommendCarouselDIV.prototype.setPosition = function(xOffset){
    this.xOffset = xOffset;
    this.parentElement.style.left = xOffset + "px";
}

RecommendCarouselDIV.prototype.setDestinationPosition = function(xOffset){
    this.destinationX = xOffset;
}

RecommendCarouselDIV.prototype.getDestinationPosition = function(){
    return this.destinationX;
}

RecommendCarouselDIV.prototype.getXOffset = function(){
    return this.xOffset;
}

UITable.prototype.addRecommendCarousel = function(baseName,carouselStyleName,carouselItemStyleName,recElements,callback,generateElement,prevBtnStyle,nextBtnStyle,animEndCallback) {
    var carousel = new WRTuiCarousel(baseName,carouselStyleName,carouselItemStyleName,recElements,callback,generateElement,prevBtnStyle,nextBtnStyle,animEndCallback);
	
    this.parentElement.appendChild(carousel.rootElement);
    return carousel;
}

function RecommendCarousel(elements, switchTime, callback, switchCallback){

    this.superParent = document.createElement('div');
    this.parentElement = document.createElement('div');
    
    this.rootElement = this.parentElement;
    this.parentElement.className = "RecommendedCarousel";
    
    this.elements = elements;
    this.currentElementDIV = new RecommendCarouselDIV();
    this.parentElement.appendChild(this.currentElementDIV.getRootElement());
    this.nextElementDIV = new RecommendCarouselDIV();
    this.parentElement.appendChild(this.nextElementDIV.getRootElement());
    
    this.superParent.appendChild(this.parentElement);
    this.callback = callback;
    this.switchTime = switchTime;
    this.position = 0;
    this.destinationPosition = 0;
    this.switchCallback = switchCallback;
    
}

RecommendCarousel.prototype.getRootElement = function(){
    return this.superParent;
}

RecommendCarousel.prototype.getCurrentElementDIV = function(){
    return this.currentElementDIV;
}
RecommendCarousel.prototype.getNextElementDIV = function(){
    return this.nextElementDIV;
}
RecommendCarousel.prototype.switchTo = function(element, thisReference, scheduleRefresh){
    if (thisReference.nextElementDIV.getRootElement().childNodes &&
    thisReference.nextElementDIV.getRootElement().childNodes.length > 0) {
        thisReference.nextElementDIV.getRootElement().removeChild(thisReference.nextElementDIV.getRootElement().childNodes[0]);
    }

    thisReference.nextElementDIV.getRootElement().appendChild(element);
    
    element.removeEventListener('click', thisReference.callback, true);
    element.addEventListener('click', thisReference.callback, true);
    
    thisReference.nextElementDIV.setPosition(thisReference.parentElement.offsetWidth);
    thisReference.currentElementDIV.setPosition(0);
    thisReference.nextElementDIV.setDestinationPosition(0);
    thisReference.currentElementDIV.setDestinationPosition(-thisReference.parentElement.offsetWidth);
    thisReference.refreshPostions(thisReference, scheduleRefresh);
    
}
RecommendCarousel.prototype.refreshPostions = function(reference, scheduleRefresh){

    if (reference.nextElementDIV.getXOffset() > 0) {
        var diff = reference.nextElementDIV.getXOffset();
        if (diff < 2) {
            reference.nextElementDIV.setPosition(0);
            reference.currentElementDIV.setPosition(-reference.parentElement.offsetWidth);
            var tmp = reference.nextElementDIV;
            reference.nextElementDIV = reference.currentElementDIV;
            reference.currentElementDIV = tmp;
        }
        else {
            reference.nextElementDIV.setPosition(reference.nextElementDIV.getXOffset() - diff / 2);
            reference.currentElementDIV.setPosition(-reference.parentElement.offsetWidth + reference.nextElementDIV.getXOffset());			
        }
            setTimeout(reference.getRefreshFunction(reference), 100);
	}
    else {
            if (scheduleRefresh) {
                if (reference.switchCallback == null) {
					clearTimeout(this.refreshTimer);
                    this.refreshTimer = setTimeout(reference.switchToNextRandomElement(reference), reference.switchTime);
                }
                else {
					clearTimeout(this.refreshTimer);
                    this.refreshTimer = setTimeout(reference.callSwitchCallback(reference), reference.switchTime);
                }
            }
    }
}

RecommendCarousel.prototype.callSwitchCallback = function(thisReference){
    return function(){
        thisReference.switchCallback.call(thisReference, thisReference);
    }
}

RecommendCarousel.prototype.getRefreshFunction = function(thisReference){
    return function(){
        thisReference.refreshPostions(thisReference, true);
    }
}
RecommendCarousel.prototype.switchToNextRandomElement = function(thisReference){
    return function(){
        thisReference.switchTo(thisReference.elements[random(thisReference.elements.length)], thisReference, true);
    }
}
RecommendCarousel.prototype.switchToRandomElement = function(thisReference, scheduleRefresh){

    thisReference.switchTo(thisReference.elements[random(thisReference.elements.length)], thisReference, scheduleRefresh);

}
//================================RECOMMEND CAROUSEL================================END

//below method should be put inside some utils file
var staticCounter = 0;
function random(number){
    staticCounter++;
    return (Math.ceil(number * Math.random()) - 1 + staticCounter) % number;
}

//================================CAROUSEL================================START

UITable.prototype.addWRTuiCarousel = function(baseName,carouselStyleName,carouselItemStyleName,recElements,callback,generateElement,prevBtnStyle,nextBtnStyle,animEndCallback){
    var carousel = new WRTuiCarousel(baseName,carouselStyleName,carouselItemStyleName,recElements,callback,generateElement,prevBtnStyle,nextBtnStyle,animEndCallback);
	
    this.parentElement.appendChild(carousel.rootElement);
    return carousel;
}

function CarouselElement(text, index, defaultSelection){
    this.xOffset = 0;
    this.index = index;
    this.selected = false;
    this.text = text;
    this.parentElement = document.createElement('div');
    this.rootElement = this.parentElement;
    
    this.updateStarSelection(defaultSelection);
    
}

CarouselElement.prototype.updateStarSelection = function(defaultSelection){
    var starHTML = '<div class="StarGray"></div>';
    if (defaultSelection == this.index) {
        starHTML = '<div class="Star"></div>';
    }
	tmpText = this.text;
	if (tmpText.length > 18) {
		tmpText = tmpText.substring(0, 17) + "...";
	}

    this.parentElement.innerHTML = starHTML + '<div class="CarouselItemText">' + tmpText + '</div>';
}

CarouselElement.prototype.getRootElement = function(){
    return this.rootElement;
}

CarouselElement.prototype.setPosition = function(xOffset){
    this.xOffset = xOffset;
    this.parentElement.style.left = xOffset + "px";
    
}

CarouselElement.prototype.getWidth = function(){
    return 262;
}

CarouselElement.prototype.setSelected = function(selected){
    this.selected = selected;
    if (selected) {
        this.parentElement.className = 'CarouselItemSelected';
    }
    else {
        this.parentElement.className = 'CarouselItem';
    }
}
UITable.prototype.addCarousel = function(elementsNames, defaultSelection, itemSelectedCallback, actionCallback){
    var carousel = new Carousel(elementsNames, defaultSelection, itemSelectedCallback, actionCallback);
    this.parentElement.appendChild(carousel.getRootElement());
    
    carousel.getLeftItem().addEventListener('click', function(event){
        carousel.selectPrevious();
    }, true);
    carousel.getRightItem().addEventListener('click', function(event){
        carousel.selectNext();
    }, true);
    carousel.getActionArea().addEventListener('click', function(event){
        carousel.action(actionCallback);
    }, true);
    carousel.doSelectionProcessing(false);
    return carousel;
}
function Carousel(elementsNames, defaultSelection, itemSelectedCallback, actionCallback){
    this.superParent = document.createElement('div');
    this.parentElement = document.createElement('div');
    this.rootElement = this.parentElement;
    this.parentElement.className = "Carousel";
    this.elements = new Array();
    for (var i = 0; i < elementsNames.length; i++) {
        var element = new CarouselElement(elementsNames[i], i, defaultSelection);
        this.elements[i] = element;
            this.parentElement.appendChild(element.getRootElement());            
    }
    this.superParent.appendChild(this.parentElement);
    
    this.position = 0;
    this.destinationPosition = 0;
    this.selectedIndex = defaultSelection > -1 ? defaultSelection : 0;
    
    this.leftItem = document.createElement("a");
    var tmpDiv = document.createElement('div');
    tmpDiv.className = "CarouselLeft";
    this.leftItem.appendChild(tmpDiv);
    this.parentElement.appendChild(this.leftItem);
    this.rightItem = document.createElement("a");
    var tmpDiv2 = document.createElement('div');
    tmpDiv2.className = "CarouselRight";
    this.rightItem.appendChild(tmpDiv2);
    this.parentElement.appendChild(this.rightItem);
    this.itemSelectedCallback = itemSelectedCallback;
    
    this.actionArea = document.createElement('div');
    this.actionArea.className = "CarouselActionArea";
    this.parentElement.appendChild(this.actionArea);
    this.actionCallback = actionCallback;

}

Carousel.prototype.getActionArea = function(){
    return this.actionArea;
}
Carousel.prototype.getRootElement = function(){
    return this.superParent;
}
Carousel.prototype.getLeftItem = function(){
    return this.leftItem;
}
Carousel.prototype.getRightItem = function(){
    return this.rightItem;
}

Carousel.prototype.refreshPostions = function(reference, callCallback){

    if (reference.destinationPosition - reference.position != 0) {
        var diff = reference.destinationPosition - reference.position;
        if (diff < 10 && diff > -10) {
            reference.position = reference.destinationPosition;
            reference.canSwitchToNextItem = true;
            if (callCallback && reference.itemSelectedCallback != null) {
                reference.itemSelectedCallback.call(reference, reference.selectedIndex);
            }            
        }
        else {
            reference.position += Math.round(diff / 2);
        }
        var elements = reference.elements;
        var currentOffset = reference.position;
        for (var i = 0; i < elements.length; i++) {
            elements[i].setPosition(currentOffset);
            currentOffset += elements[i].getWidth();
        }
        
        setTimeout(reference.getRefreshFunction(reference, callCallback), 100);
    }
}
Carousel.prototype.getRefreshFunction = function(reference, callCallback){
    return function(){
            reference.refreshPostions(reference, callCallback);
    };
}
Carousel.prototype.getCenter = function(){
    return window.innerWidth / 2;
}

Carousel.prototype.calculateOffsetToCenterSelected = function(){
    var elements = this.elements;
    var currentOffset = this.getCenter();//this.position;
    for (var i = 0; i < elements.length; i++) {
        if (i == this.selectedIndex) {
            currentOffset -= elements[i].getWidth() / 2;
            this.destinationPosition = currentOffset;
            return;
        }
        else {
            currentOffset -= elements[i].getWidth();
        }
    }
}

Carousel.prototype.correctSelection = function(){
    var elements = this.elements;
    for (var i = 0; i < elements.length; i++) {
        elements[i].setSelected(i == this.selectedIndex);
    }
}

Carousel.prototype.removeDefaultSelection = function(){
    var elements = this.elements;
    for (var i = 0; i < elements.length; i++) {
        elements[i].updateStarSelection(-1);
    }
    this.actionCallback.call(this, -1);
}


Carousel.prototype.action = function(callCallback){
    var elements = this.elements;
    for (var i = 0; i < elements.length; i++) {
        elements[i].updateStarSelection(this.selectedIndex);
    }
    callCallback.call(this, this.selectedIndex);
}

Carousel.prototype.doSelectionProcessing = function(callCallback){
    this.correctSelection();
    this.calculateOffsetToCenterSelected();
    this.refreshPostions(this, callCallback);
}

Carousel.prototype.selectNext = function(event){
    if (this.selectedIndex < this.elements.length - 1 && this.canSwitchToNextItem) {
        this.selectedIndex++;
    }
    else {
        this.selectedIndex = 0;
    }
    this.canSwitchToNextItem = false;
    this.doSelectionProcessing(true);
}

Carousel.prototype.selectPrevious = function(event){
    if (this.selectedIndex > 0 && this.canSwitchToNextItem) {
        this.selectedIndex--;
        
    }
    else {
        this.selectedIndex = this.elements.length - 1;
    }
    this.canSwitchToNextItem = false;
    this.doSelectionProcessing(true);
    
}
//================================CAROUSEL================================END


// Helper functions begin

function eventTest(e){
    alert("Event test: " + getKeys(e.srcElement));
    return true;
}

// Helper functions end

function formTest(e){
    alert(searchInput.value);
    return true;
}

/*

 document.addEventListener('mousedown', setStartY, false);

 document.addEventListener('mouseup', startScroll, false);

 */

