/*
   ********************************************************
   Nokia Copyright 2010
   
   fileLoader.js
   
   This file contains a class for synchronous dynamic loading
   of javascript (and stylesheet) files, any file that is
   loaded by the fileLoader class MUST have "fL.poll();"
   on the last line of code, otherwise the fileLoader wont
   continue to load the files that are in the queue.
   
   When this file is added to a HTML (preferably in the top),
   it creates a global class object "fL" which can then be 
   called from any script.
   ********************************************************
*/

/*
   *****************************************
   Function used to bypass the browser cache
   *****************************************
*/
function nocache(url) {
    if (url.indexOf("?") == -1) {
        url += "?";
    } else {
        url += "&";
    }
    url += "nocache=" + (new Date().getTime());
	return url;
}

/*
   *******************************************************
   Function for the main class
   *******************************************************
*/
function fileLoader() {
	this.init();
}

/*
   *******************************************************
   Set up properties for the class
   *******************************************************
*/
fileLoader.prototype.fileQ = null;
fileLoader.prototype.fileQcnt = 0;

/*
   *******************************************************
   Function for initialising when creating the class
   *******************************************************
*/
fileLoader.prototype.init = function() {
	this.fileQ = new Array();
}

/*
   *******************************************************
   Function for adding files to the queue
   *******************************************************
*/
fileLoader.prototype.add = function(fileName,callback) {
	qObj = new Array();
	qObj[0] = fileName;
	
	if (callback!=undefined) {
		qObj[1] = callback;
	} else {
		qObj[1] = function() { fL.poll(); }
	}
	this.fileQ.push(qObj);

//	alert('Adding '+fileName+' to the queue.');
}

/*
   *******************************************************
   Function for polling the queue and loading the file that
   is next in line
   *******************************************************
*/
fileLoader.prototype.poll = function() {
	if (this.fileQ.length > 0) {
		var qObj = this.fileQ.shift();
		var src = qObj[0];
		var callback = qObj[1];
		
		var head = document.getElementsByTagName('head')[0],
			script = document.createElement('script');
		done = false;
		script.setAttribute('src', src);
		script.setAttribute('type', 'text/javascript');
		script.setAttribute('charset', 'utf-8');
		script.onload = script.onreadstatechange = function() {
			if (!done && (!this.readyState || this.readyState == 'loaded' || this.readyState == 'complete')) {
				done = true;
				script.onload = script.onreadystatechange = null;
					if (callback) {
						callback();
					}
				}
		}
		head.insertBefore(script, head.firstChild);
	}

}

/*
   *******************************************************
   Function for showing a message for each file that is loaded.
   This function should be overloaded either with null or with
   a customized alert display function.
   *******************************************************
*/
fileLoader.prototype.fAlert = function(alertMsg) {
	logMsg(alertMsg);
//	alert(alertMsg + '\n\nPlease overload this function fL.fAlert(); with null or something useful to indicate progress of loading the scripts.');
}

// Create the global class object
var fL = new fileLoader();
